@@ -154,25 +154,25 @@
 				      (or (not func) 
 					  (cadr (func (labels ellipsis-label))))))))))))
 	     
-	   (define (handle-regex reg e)
-	     (lambda (x)
-	       (and (string? x)
-		    (with-let (sublet (symbol->value '*libc* e)
-				:x x
-				:regexp (substring reg 1 (- (length reg) 1)))
-		      (let* ((rg (regex.make))
-			     (res (regcomp rg regexp 0)))
-			(unless (zero? res)
-			  (error 'regex-error "~S~%" (regerror res rg)))
-			(set! res (regexec rg x 0 0))
-			(regfree rg)
-			(regex.free rg)
-			(zero? res))))))
+	   (define handle-regex
+	     (let ((rg ((*libc* 'regex.make))) ; is this safe?
+		   (local-regcomp (*libc* 'regcomp))
+		   (local-regerror (*libc* 'regerror))
+		   (local-regexec (*libc* 'regexec))
+		   (local-regfree (*libc* 'regfree)))
+	       (lambda (reg e)
+		 (lambda (x)
+		   (and (string? x)
+			(let ((res (local-regcomp rg (substring reg 1 (- (length reg) 1)) 0)))
+			  (unless (zero? res)
+			    (error 'regex-error "~S~%" (local-regerror res rg)))
+			  (set! res (local-regexec rg x 0 0))
+			  (local-regfree rg)
+			  (zero? res)))))))
 
 	   (define (undefined->function undef e)   ; handle the pattern descriptor ("undef") of the form #< whatever >, "e" = caller's curlet
 	     (let* ((str1 (object->string undef))
 		    (str1-end (- (length str1) 1)))
-	       ;(format *stderr* "~S ~S ~S~%" undef str1 str1-end)
 	       (if (not (char=? (str1 str1-end) #\>))
 		   (error 'wrong-type-arg "pattern descriptor does not end in '>': ~S\n" str1))
 	       (let ((str (substring str1 2 str1-end)))
@@ -335,6 +335,10 @@
 				    (and (undefined? target)              ; #<...>
 					 (not (eq? target #<undefined>))
 					 (let ((func (undefined->function target e)))
+					   ;(format *stderr* "func: ~S~%" func)
+					   ;; (if (undefined? func) (error 'unbound-variable "function ~A is undefined\n" str))
+					   ;; not the above check because we want to be able to pass patterns as selectors! (scase37 in s7test)
+					   ;;    this seems like a mistake: #<symbol?> won't work? 
 					   (and (procedure? func)
 						(func select))))
 				    (and (sequence? target)
@@ -501,8 +505,8 @@
 	    (lambda (x)
 	      (or (null? x)
 		  (and (pair? x)
-		       (and (symbol? (car x))
-			    (symbols? (cdr x))))))))
+                       (symbol? (car x))
+		       (symbols? (cdr x)))))))
   (case* x
     ((#<symbols?>) #t)
     (else #f))))
