@@ -48,7 +48,7 @@
 			    (y1 (e (+ i 3))))
 			(if (< (abs (real-part (- y0 y1))) .0001)
 			    (/ (- x1 x0) (* y0 all-x))
-			    (/ (* (- (log y1) (log y0)) 
+			    (/ (* (log (/ y1 y0)) ; (- (log y1) (log y0)) 
 				  (- x1 x0)) 
 			       (* (- y1 y0) all-x))))))
 	    ;; or (* (/ (- (log y1) (log y0)) (- y1 y0)) 
@@ -118,9 +118,10 @@
 	  (do ((i 0 (+ i 1))
 	       (sum 0.0 0.0))
 	      ((= i N))
-	    (do ((k 0 (+ k 1)))
+	    (do ((k 0 (+ k 1))
+		 (cin (/ (* 2.0 0+1.0i pi i) N)))
 		((= k N))
-	      (set! sum (+ sum (* (vals k) (exp (/ (* 2.0 0+1.0i pi k i) N))))))
+	      (set! sum (+ sum (* (vals k) (exp (* cin k))))))
 	    (set! (w i) (magnitude sum))
 	    (set! pk (max pk (w i))))
 	  ;; scale to 1.0 (it's usually pretty close already, that is pk is close to 1.0)
@@ -339,7 +340,7 @@ squeezing in the frequency domain, then using the inverse DFT to get the time do
 	      (data1 (make-float-vector (+ len 1))))
 	  (copy data data1 1)
 	  (float-vector-abs! (float-vector-multiply! data1 data))
-	  (float-vector-multiply! data (make-shared-vector data1 (list len) 1))
+	  (float-vector-multiply! data (subvector data1 1 (+ len 1)))
 	  (let ((amp1 (/ amp (float-vector-peak data))))
 	    (float-vector->channel (float-vector-scale! data amp1) 0 len snd chn current-edit-position "spike")))))))
 
@@ -438,15 +439,14 @@ squeezing in the frequency domain, then using the inverse DFT to get the time do
 	(let ((fftscale (/ 1.0 fftlen))
 	      (old-pk (float-vector-peak rl))
 	      (im (make-float-vector fftlen)))
-	  (if (> old-pk 0.0)
-	      (begin
-		(fft rl im 1)
-		(rectangular->magnitudes rl im)
-		(float-vector-scale! rl fftscale)
-		(float-vector-scale! im 0.0)
-		(fft rl im -1)
-		(let ((pk (float-vector-peak rl)))
-		  (float-vector->channel (float-vector-scale! rl (/ old-pk pk)) 0 len snd chn #f "zero-phase")))))))))
+	  (when (> old-pk 0.0)
+	    (fft rl im 1)
+	    (rectangular->magnitudes rl im)
+	    (float-vector-scale! rl fftscale)
+	    (float-vector-scale! im 0.0)
+	    (fft rl im -1)
+	    (let ((pk (float-vector-peak rl)))
+	      (float-vector->channel (float-vector-scale! rl (/ old-pk pk)) 0 len snd chn #f "zero-phase"))))))))
 
 (define rotate-phase
   (let ((+documentation+ "(rotate-phase func snd chn) calls fft, applies func to each phase, then un-ffts"))
@@ -458,23 +458,22 @@ squeezing in the frequency domain, then using the inverse DFT to get the time do
 	      (fftscale (/ 1.0 fftlen))
 	      (old-pk (float-vector-peak rl))
 	      (im (make-float-vector fftlen)))
-	  (if (> old-pk 0.0)
-	      (begin
-		(fft rl im 1)
-		(rectangular->magnitudes rl im)
-		(float-vector-scale! rl fftscale)
-		(set! (im 0) 0.0)
-		(do ((i 1 (+ i 1))
-		     (j (- fftlen 1) (- j 1)))
-		    ((= i fftlen2))
-		  ;; rotate the fft vector by func, keeping imaginary part complex conjgate of real
-		  (set! (im i) (func (im i)))
-		  (set! (im j) (- (im i))))
-		(polar->rectangular rl im)
-		(fft rl im -1)
-		(let ((pk (float-vector-peak rl)))
-		  (float-vector->channel (float-vector-scale! rl (/ old-pk pk)) 0 len snd chn #f 
-					 (format #f "rotate-phase ~A" (procedure-source func)))))))))))
+	  (when (> old-pk 0.0)
+	    (fft rl im 1)
+	    (rectangular->magnitudes rl im)
+	    (float-vector-scale! rl fftscale)
+	    (set! (im 0) 0.0)
+	    (do ((i 1 (+ i 1))
+		 (j (- fftlen 1) (- j 1)))
+		((= i fftlen2))
+	      ;; rotate the fft vector by func, keeping imaginary part complex conjgate of real
+	      (set! (im i) (func (im i)))
+	      (set! (im j) (- (im i))))
+	    (polar->rectangular rl im)
+	    (fft rl im -1)
+	    (let ((pk (float-vector-peak rl)))
+	      (float-vector->channel (float-vector-scale! rl (/ old-pk pk)) 0 len snd chn #f 
+					 (format #f "rotate-phase ~A" (procedure-source func))))))))))
 
 #|
 (rotate-phase (lambda (x) 0.0)) is the same as (zero-phase)
@@ -1543,11 +1542,11 @@ shift the given channel in pitch without changing its length.  The higher 'order
 	  (set! *output* summer)
 	  (do ((i 0 (+ i 1)))
 	      ((= i pairs))
-	    (do ((gen (vector-ref ssbs i))
-		 (filt (vector-ref bands i))
-		 (k 0 (+ k 1)))
-		((= k len))
-	      (outa k (ssb-am gen (bandpass filt (ina k data)))))) ; outa adds, (ina i v) is the same as (float-vector-ref v i)
+	    (let ((gen (vector-ref ssbs i))
+		  (filt (vector-ref bands i)))
+	      (do ((k 0 (+ k 1)))
+		  ((= k len))
+		(outa k (ssb-am gen (bandpass filt (ina k data))))))) ; outa adds, (ina i v) is the same as (float-vector-ref v i)
 	  (set! *output* #f)
 	  (float-vector-scale! summer (/ mx (float-vector-peak summer)))
 	  (float-vector->channel summer beg len snd chn current-edit-position
@@ -1581,12 +1580,12 @@ shift the given channel in pitch without changing its length.  The higher 'order
 	  (set! *output* summer)
 	  (do ((i 0 (+ i 1)))
 	      ((= i pairs))
-	    (do ((gen (vector-ref ssbs i))
-		 (filt (vector-ref bands i))
-		 (e (vector-ref frenvs i))
-		 (k 0 (+ k 1)))
-		((= k len))
-	      (outa k (ssb-am gen (bandpass filt (ina k data)) (env e)))))
+	    (let ((gen (vector-ref ssbs i))
+		  (filt (vector-ref bands i))
+		  (e (vector-ref frenvs i)))
+	      (do ((k 0 (+ k 1)))
+		  ((= k len))
+		(outa k (ssb-am gen (bandpass filt (ina k data)) (env e))))))
 	  (set! *output* #f)
 	  (float-vector-scale! summer (/ mx (float-vector-peak summer)))
 	  (float-vector->channel summer beg len snd chn current-edit-position
@@ -1652,19 +1651,18 @@ shift the given channel in pitch without changing its length.  The higher 'order
 	    (do ((i 0 (+ i 1)))
 		((= i fft-len))
 	      (float-vector-set! newv i (mus-random dither)))))
-      (if (> num-coeffs 1)
-	  (begin
-	    (float-vector-add! newv (float-vector-scale! (copy sound) (coeffs 1)))
-	    (if (> num-coeffs 2)
-		(let ((peak (maxamp snd chn)))
-		  (copy sound rl1)
-		  (do ((i 2 (+ i 1)))
-		      ((= i num-coeffs))
-		    (copy sound rl2)
-		    (convolution rl1 rl2 fft-len)
-		    (float-vector-add! newv (float-vector-scale! (copy rl1) 
-								 (/ (* (coeffs i) peak) (float-vector-peak rl1)))))
-		  (float-vector-scale! newv (/ peak (float-vector-peak newv)))))))
+      (when (> num-coeffs 1)
+	(float-vector-add! newv (float-vector-scale! (copy sound) (coeffs 1)))
+	(if (> num-coeffs 2)
+	    (let ((peak (maxamp snd chn)))
+	      (copy sound rl1)
+	      (do ((i 2 (+ i 1)))
+		  ((= i num-coeffs))
+		(copy sound rl2)
+		(convolution rl1 rl2 fft-len)
+		(float-vector-add! newv (float-vector-scale! (copy rl1) 
+							     (/ (* (coeffs i) peak) (float-vector-peak rl1)))))
+	      (float-vector-scale! newv (/ peak (float-vector-peak newv))))))
       (float-vector->channel newv 0 (max len (* len (- num-coeffs 1))) 
 			     snd chn #f 
 			     (format #f "spectral-polynomial ~A" (float-vector->string coeffs))))))
@@ -1727,13 +1725,12 @@ the rendering frequency, the number of measurements per second; 'db-floor' is th
 	      (do ((j 0 (+ j 1)))
 		  ((= j fftsize))
 		(float-vector-set! fdr j (readin rd)))
-	      (if (>= (linear->db (sqrt (/ (dot-product fdr fdr) fftsize))) db-floor)
-		  (begin
-		    (fill! fdi 0.0)
-		    (mus-fft fdr fdi fftsize)
-		    (rectangular->magnitudes fdr fdi)
-		    (set! (results loc) (/ (dot-product scl fdr fft2) 
-					   (dot-product ones fdr fft2))))))
+	      (when (>= (linear->db (sqrt (/ (dot-product fdr fdr) fftsize))) db-floor)
+		(fill! fdi 0.0)
+		(mus-fft fdr fdi fftsize)
+		(rectangular->magnitudes fdr fdi)
+		(set! (results loc) (/ (dot-product scl fdr fft2) 
+				       (dot-product ones fdr fft2)))))
 	    results))))))
 
 
@@ -2016,7 +2013,7 @@ and replaces it with the spectrum given in coeffs")
 	       (axis-x1 (axinfo 12))
 	       (axis-y1 (axinfo 11))
 	       (bark-label-font (snd-font 3))
-	       (cr (make-cairo (car (channel-widgets snd chn))))
+	       (cr ((channel-widgets snd chn) 17))
 	       (scale-position (let ((sr2 (* 0.5 (srate snd))))
 				 (lambda (scale f)
 				   (let ((b20 (scale 20.0)))
@@ -2081,19 +2078,17 @@ and replaces it with the spectrum given in coeffs")
 	    (set! (foreground-color snd chn copy-context) color3)
 	    (if (= bark-tick-function 2) (draw-bark-ticks erb-position))
 	    (if bark-label-font (set! (current-font snd chn copy-context) bark-label-font))
-	    (draw-string "erb" (+ (* char-width 11) label-pos) (+ axis-y1 label-height) snd chn copy-context cr)
-	    (free-cairo cr))
+	    (draw-string "erb" (+ (* char-width 11) label-pos) (+ axis-y1 label-height) snd chn copy-context cr))
 	  
 	  (set! (foreground-color snd chn copy-context) old-foreground-color)))
     
       ;; mouse click = move to next scale's ticks
       (define (choose-bark-ticks hook)
-	(if (= (hook 'axis) lisp-graph)
-	    (begin
-	      (set! bark-tick-function (+ bark-tick-function 1))
-	      (if (> bark-tick-function 2)
-		  (set! bark-tick-function 0))
-	      (update-lisp-graph (hook 'snd) (hook 'chn)))))
+	(when (= (hook 'axis) lisp-graph)
+	  (set! bark-tick-function (+ bark-tick-function 1))
+	  (if (> bark-tick-function 2)
+	      (set! bark-tick-function 0))
+	  (update-lisp-graph (hook 'snd) (hook 'chn))))
       
       ;; user's view of display-bark-fft function
       (lambda* (off col1 col2 col3)
@@ -2178,18 +2173,17 @@ is assumed to be outside -1.0 to 1.0."))
 	     (j (- n 1) (- j 1)))
 	    ((= i m))
 	  (set! (reg i) (data j)))
-	(do ((j 0 (+ j 1)))
+	(do ((j 0 (+ j 1))
+	     (sum (dot-product coeffs reg m) (dot-product coeffs reg m)))
 	    ((= j nf) future)
-	  (let ((sum (dot-product coeffs reg m)))
-	    (do ((k (- m 1) (- k 1)))
-		((= k 0))
-	      (set! (reg k) (reg (- k 1))))
-	    
-	    ;; added this block
-	    (if clipped
-		(set! sum (if (> sum 0.0) (max sum 1.0) (min sum -1.0))))
-	    (set! (reg 0) sum)
-	    (set! (future j) sum)))))))
+	  (do ((k (- m 1) (- k 1)))
+	      ((= k 0))
+	    (set! (reg k) (reg (- k 1))))
+	  ;; added this block
+	  (if clipped
+	      (set! sum (if (> sum 0.0) (max sum 1.0) (min sum -1.0))))
+	  (set! (reg 0) sum)
+	  (set! (future j) sum))))))
 
 
 ;;; -------- unclip-channel
@@ -2210,23 +2204,21 @@ is assumed to be outside -1.0 to 1.0."))
 	  (do ((i 0 (+ i 1)))
 	      ((= i len))
 	    (if (> (float-vector-ref data i) .9999)                    ; this sample is clipped
-		(if (not in-clip)
-		    (begin
-		      (set! in-clip #t)
-		      (set! clip-beg i)))
+		(unless in-clip
+		  (set! in-clip #t)
+		  (set! clip-beg i))
 		(begin                            ; not clipped
 		  (set! unclipped-max (max unclipped-max (float-vector-ref data i))) ; this is not the same as (float-vector-peak ...)
-		  (if in-clip
-		      (begin
-			(set! in-clip #f)
-			(set! (clip-data clips) clip-beg)
-			(set! (clip-data (+ 1 clips)) (- i 1))
-			(set! clips (+ clips 2))
-			(if (> clips clip-size)
-			    (let ((new-clip-data (make-vector (* 2 clip-size) 0)))
-			      (copy clip-data new-clip-data)
-			      (set! clip-data new-clip-data)
-			      (set! clip-size (* 2 clip-size))))))))))
+		  (when in-clip
+		    (set! in-clip #f)
+		    (set! (clip-data clips) clip-beg)
+		    (set! (clip-data (+ 1 clips)) (- i 1))
+		    (set! clips (+ clips 2))
+		    (if (> clips clip-size)
+			(let ((new-clip-data (make-vector (* 2 clip-size) 0)))
+			  (copy clip-data new-clip-data)
+			  (set! clip-data new-clip-data)
+			  (set! clip-size (* 2 clip-size)))))))))
 	
 	(if (<= clips 0)
 	    'no-clips
@@ -2358,7 +2350,7 @@ is assumed to be outside -1.0 to 1.0."))
 					;(format () "~%~%invert-matrix n: ~D, ~S, b: ~A, ~S~%" (length matrix) matrix (and b (length b)) b)
       (call-with-exit
        (lambda (return)
-	 (let ((n (car (vector-dimensions matrix))))
+	 (let ((n (vector-dimension matrix 0)))
 	   (let ((cols (make-vector n 0))
 		 (rows (make-vector n 0))
 		 (pivots (make-vector n 0)))
@@ -2376,20 +2368,18 @@ is assumed to be outside -1.0 to 1.0."))
 			 ((= k n))
 		       (if (= (pivots k) 0)
 			   (let ((val (abs (matrix j k))))
-			     (if (> val biggest)
-				 (begin
-				   (set! col k)
-				   (set! row j)
-				   (set! biggest val))))
+			     (when (> val biggest)
+			       (set! col k)
+			       (set! row j)
+			       (set! biggest val)))
 			   (if (> (pivots k) 1)
 			       (return #f))))))
 	       (set! (pivots col) (+ (pivots col) 1))
 	       (if (not (= row col))
 		   (let ((temp (if (sequence? b) (b row) 0.0)))
-		     (if (sequence? b)
-			 (begin
-			   (set! (b row) (b col))
-			   (set! (b col) temp)))
+		     (when (sequence? b)
+		       (set! (b row) (b col))
+		       (set! (b col) temp))
 		     (do ((k 0 (+ k 1)))
 			 ((= k n))
 		       (set! temp (matrix row k))
@@ -2658,10 +2648,9 @@ the multi-modulator FM case described by the list of modulator frequencies and i
 		(if (> (random 1.0) 0.5)
 		    (set! (new-partials k) (- (new-partials k)))))
 	      (let ((new-sum (cos-fft-to-max topk new-partials)))
-		(if (< new-sum min-sum)
-		    (begin
-		      (set! min-partials (copy new-partials))
-		      (set! min-sum new-sum))))))
+		(when (< new-sum min-sum)
+		  (set! min-partials (copy new-partials))
+		  (set! min-sum new-sum)))))
 	  
 	  (let ((new-amps (float-vector-scale! min-partials (/ original-sum min-sum)))
 		(new-partials (copy partials)))
