@@ -112,7 +112,9 @@
 		       'format             (with-mock-wrapper* #_format)
 		       'write              (with-mock-wrapper* #_write)
 		       'display            (with-mock-wrapper* #_display)
+		       'vector-dimension   (with-mock-wrapper #_vector-dimension)
 		       'vector-dimensions  (with-mock-wrapper #_vector-dimensions)
+		       'vector-rank        (with-mock-wrapper #_vector-rank)
 		       'fill!              (with-mock-wrapper* #_fill!)
 		       'vector-fill!       (with-mock-wrapper* #_vector-fill!)
 		       'vector->list       (with-mock-wrapper* #_vector->list)
@@ -167,6 +169,20 @@
 	 'let-ref-fallback local-ref
 	 'vector-set! local-set!
 	 'let-set-fallback local-set!))))
+
+  ;; vector with data (mimicking Clojure):
+  (define (meta-vector v data)
+    (openlet
+     (sublet (*mock-vector* 'mock-vector-class)
+       'value v
+       'mock-type 'mock-vector?
+       'meta-data data)))
+  
+  (define v (meta-vector #(0 1 2) "hiho"))
+  (v 1): 1
+  (v 'meta-data): "hiho"
+  v: #(0 1 2)
+  (vector? v): #t
 |#
 
 
@@ -285,10 +301,10 @@
 						     (if (string? str)
 							 (length str)
 							 (error 'wrong-type-arg "string-length argument should be a string: ~A" str)))
-						   (with-mock-wrapper #_string-length))
+						   (with-mock-wrapper #_length))
 
 		       'string-append          (with-mock-wrapper* #_string-append)
-		       'string-copy            (with-mock-wrapper #_copy) ; new form -> with-mock-wrapper* ? 
+		       'string-copy            (with-mock-wrapper* #_string-copy)
 		       
 		       'string=?               (with-mock-wrapper* #_string=?)
 		       'string<?               (with-mock-wrapper* #_string<?)
@@ -299,7 +315,7 @@
 		       'string-downcase        (with-mock-wrapper #_string-downcase)
 		       'string-upcase          (with-mock-wrapper #_string-upcase)
 		       'string->symbol         (with-mock-wrapper #_string->symbol)
-		       'symbol                 (with-mock-wrapper #_symbol)
+		       'symbol                 (with-mock-wrapper* #_symbol)
 		       'string->keyword        (with-mock-wrapper #_string->keyword)
 		       'open-input-string      (with-mock-wrapper #_open-input-string)
 		       'directory?             (with-mock-wrapper #_directory?)
@@ -320,7 +336,7 @@
 		       'load                   (with-mock-wrapper* #_load)
 		       'eval-string            (with-mock-wrapper* #_eval-string)
 		       'string->list           (with-mock-wrapper* #_string->list)
-		       'bignum                 (with-mock-wrapper #_bignum)
+		       'bignum                 (with-mock-wrapper* #_bignum)
 		       'fill!                  (with-mock-wrapper* #_fill!)
 		       'write-string           (with-mock-wrapper* #_write-string)
 		       'copy                   (with-mock-wrapper* #_copy)
@@ -335,7 +351,7 @@
 		       'string-ci<=?           (with-mock-wrapper* #_string-ci<=?)
 		       'string-ci>=?           (with-mock-wrapper* #_string-ci>=?)
 		       'string?                (with-mock-wrapper #_string?)
-		       'length                 (with-mock-wrapper #_string-length)
+		       'length                 (with-mock-wrapper (if (provided? 'pure-s7) #_length #_string-length))
 		       'append                 (with-mock-wrapper* #_append)
 		       'class-name             '*mock-string*)))
 	  
@@ -433,7 +449,7 @@
 		  (sublet (*mock-char* 'mock-char-class)
 		    'value c
 		    'mock-type 'mock-char?)))
-		(error 'wrong-type-arg "mock-char arg ~S is not a char" c)))
+		(error 'wrong-type-arg "mock-char argument ~S is not a char" c)))
 	  
 	  (set! mock-char? (lambda (obj)
 			     (and (let? obj)
@@ -509,6 +525,8 @@
 		 'real?            (with-mock-wrapper #_real?)
 		 'complex?         (with-mock-wrapper #_complex?)
 		 'rational?        (with-mock-wrapper #_rational?)
+		 'byte?            (with-mock-wrapper #_byte?)
+		 'float?           (with-mock-wrapper #_float?)
 		 'exact?           (with-mock-wrapper #_exact?)
 		 'inexact?         (with-mock-wrapper #_inexact?)
 		 'lognot           (with-mock-wrapper #_lognot)
@@ -582,7 +600,6 @@
 		 'subvector        (with-mock-wrapper* #_subvector)
 		 'read-string      (with-mock-wrapper* #_read-string)
 		 'length           (with-mock-wrapper #_length)
-		 'number?          (with-mock-wrapper #_number?)
 		 'class-name       '*mock-number*)))
 	  
 	  (define (mock-number x)
@@ -780,7 +797,7 @@
 								(#_list-ref (obj 'value) ind))))
 						     (openlet obj) 
 						     val)
-						   (error "let-ref mock-pair index is not an integer: ~S" ind))))
+						   (error 'wrong-type-arg "let-ref mock-pair index is not an integer: ~S" ind))))
 		       'let-set-fallback (lambda (obj ind val) 
 					   (if (eq? ind 'value)
 					       #<undefined>
@@ -790,7 +807,7 @@
 								(#_list-set! (obj 'value) ind val))))
 						     (openlet obj)
 						     val)
-						   (error "let-set! mock-pair index is not an integer: ~S" ind))))
+						   (error 'wrong-type-arg "let-set! mock-pair index is not an integer: ~S" ind))))
 		       
 		       'reverse!         (lambda (obj) 
 					   (if (mock-pair? obj)
@@ -872,10 +889,10 @@
 		       'symbol->string        (with-mock-wrapper #_symbol->string)
 		       'symbol->value         (with-mock-wrapper* #_symbol->value)
 		       'symbol->dynamic-value (with-mock-wrapper #_symbol->dynamic-value)
-		       'setter                (with-mock-wrapper #_setter)
+		       'setter                (with-mock-wrapper* #_setter)
 		       'provided?             (with-mock-wrapper #_provided?)
 		       'provide               (with-mock-wrapper #_provide)
-		       'defined?              (with-mock-wrapper #_defined?)
+		       'defined?              (with-mock-wrapper* #_defined?)
 		       'symbol->keyword       (with-mock-wrapper #_symbol->keyword)
 		       'keyword?              (with-mock-wrapper #_keyword?)
 		       'keyword->symbol       (with-mock-wrapper #_keyword->symbol)
@@ -956,7 +973,7 @@
 	    (immutable!
 	     (openlet 
 	      (sublet (*mock-random-state* 'mock-random-state-class)
-		'value (#_random-state seed carry)
+		'value (if (provided? 'gmp) (#_random-state seed) (#_random-state seed carry))
 		'mock-type 'mock-random-state?))))
 	  
 	  (set! mock-random-state? 
