@@ -1,6 +1,7 @@
-/* nrepl, notcurses-base repl
+/* nrepl, notcurses-based repl
  *
- * gcc -o nrepl nrepl.c s7.o -Wl,-export-dynamic -lnotcurses -lm -I. -ldl
+ * gcc -o nrepl nrepl.c s7.o -Wl,-export-dynamic -lnotcurses-core -lm -I. -ldl
+ *   (s7.o: gcc -c s7.c -o s7.o -I. -O2 -g)
  */
 
 #include <stdio.h>
@@ -323,9 +324,10 @@ static int nrepl(s7_scheme *sc)
 	  return(0);
 	}
       fprintf(stderr, "load %s\n", argv[1]);
+      errno = 0;
       if (!s7_load(sc, argv[1]))
 	{
-	  fprintf(stderr, "can't load %s\n", argv[1]);
+	  fprintf(stderr, "%s: %s\n", strerror(errno), argv[1]);
 	  return(2);
 	}
     }
@@ -368,12 +370,10 @@ static int nrepl(s7_scheme *sc)
 #if 0
 /*
   gcc -c s7.c -O2 -I. -Wl,-export-dynamic -lm -ldl
-  gcc -o nrepl nrepl.c s7.o -lnotcurses -lm -I. -ldl
+  gcc -o nrepl nrepl.c s7.o -lnotcurses-core -lm -I. -ldl
 
   To build s7 to nrepl in one line:
-    gcc -o nrepl s7.c -O2 -I. -Wl,-export-dynamic -lm -ldl -DWITH_MAIN -DWITH_NOTCURSES -lnotcurses
-
-  In version 2.1.6(7?) add -lnotcurses-core after -lnotcurses
+    gcc -o nrepl s7.c -O2 -I. -Wl,-export-dynamic -lm -ldl -DWITH_MAIN -DWITH_NOTCURSES -lnotcurses-core
 
   nrepl-bits.h is generated from (make-nrepl-bits.scm):
 
@@ -390,5 +390,17 @@ static int nrepl(s7_scheme *sc)
 	  (if (char=? c #\newline)
 	      (format op "~%  ")))))))
 
+  xxd can create the array, but you need to tack on the trailing 0
+  Christos Vagias suggests:
+
+    xxd -i < scheme_file.scm > scheme_file.xxd; echo ", 0x00" >>  scheme_file.xxd
+    and scheme_file.xxd will look like
+      0x01, 0x02, 0xAA,.....
+      , 0x00
+    And then in code
+      const char main_scm[] = {
+      #include "./resources/main_scm.xxd"
+      };
+
 */
 #endif
