@@ -47,11 +47,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "14-Oct-21"
+#define SND_DATE "8-Feb-22"
 #ifndef SND_VERSION
-#define SND_VERSION "21.8"
+#define SND_VERSION "22.1"
 #endif
-#define SND_MAJOR_VERSION "21"
-#define SND_MINOR_VERSION "8"
+#define SND_MAJOR_VERSION "22"
+#define SND_MINOR_VERSION "1"
 
 #endif
