@@ -110,7 +110,7 @@
 	      (if (pair? (cddr val))
 		  `(set! (setter ',(car val))
 			 (if (not (,(caddr val) ,(car val))) ; check initial value (already set)
-			     (error 'wrong-type "initial value ~S is not ~S" ,(car val) ,(caddr val))
+			     (error 'wrong-type "typed-let: initial value ~S is not ~S" ,(car val) ,(caddr val))
 			     ,(caddr val)))                  ; assume built-in type here
 		  (values)))
 	    vals)
@@ -153,8 +153,8 @@
 	(let ((result (if (= (car (arity bp)) 1)
 			  (type e)
 			  (bp 'the e))))
-	  (if result e (error 'bad-type "~S is ~S but should be ~S" e (type-of e) bp)))
-	(error 'bad-type "~S is not a boolean procedure" bp))))
+	  (if result e (error 'bad-type "~S is ~S but should be ~S in (the ~S ~S)" e (type-of e) bp type expr)))
+	(error 'bad-type "~S is not a boolean procedure, (the ~S ~S)" bp type expr))))
 
 (define iota
   (let ((+documentation+ "(iota n (start 0) (incr 1)) returns a list counting from start for n:\n\
@@ -244,19 +244,20 @@
 ;;; ----------------
 (define-macro (fully-macroexpand form)
   (define (expand form)
-    (if (pair? form)
-	(if (and (symbol? (car form))
-		 (macro? (symbol->value (car form))))
-	    (expand (apply macroexpand (list form)))
-	    (if (and (eq? (car form) 'set!)
-		     (pair? (cdr form))
-		     (pair? (cadr form))
-		     (macro? (symbol->value (caadr form))))
-		(expand (apply macroexpand (list (cons (setter (symbol->value (caadr form)))
-						       (append (cdadr form) (copy (cddr form)))))))
-		(cons (expand (car form))
-		      (expand (cdr form)))))
-	form))
+    (cond ((not (pair? form)) 
+	   form)
+          ((and (symbol? (car form))
+                (macro? (symbol->value (car form))))
+           (expand (apply macroexpand (list form))))
+          ((and (eq? (car form) 'set!)
+                (pair? (cdr form))
+                (pair? (cadr form))
+                (macro? (symbol->value (caadr form))))
+           (expand (apply macroexpand (list (cons (setter (symbol->value (caadr form))) 
+						  (append (cdadr form) (copy (cddr form))))))))
+          (else 
+	   (cons (expand (car form)) 
+		 (expand (cdr form))))))
   (list 'quote (expand form)))
 
 (define-macro (define-with-macros name&args . body)
@@ -979,37 +980,38 @@ Unlike full-find-if, safe-find-if can handle any circularity in the sequences.")
 				 (cons (car slot) (cadr slot))
 				 (cons slot #f)))
 			   ,slots)                    ; the incoming new slots, #f is the default value
-		      new-slots))))                   ; the inherited slots
+		      new-slots)))                    ; the inherited slots
 
-     (set! new-methods
-	   (append (map (lambda (method)
-			  (if (pair? method)
-			      (cons (car method) (cadr method))
-			      (cons method #f)))
-			,methods)                     ; the incoming new methods
-
-		   ;; add an object->string method for this class (this is already a generic function).
-		   (list (cons 'object->string
-			       (lambda (obj . rest)
-				 (if (and (pair? rest)
-					  (eq? (car rest) :readable))    ; write readably
-				     (format #f "(make-~A~{ :~A ~W~^~})"
-					     ',class-name
-					     (map (lambda (slot)
-						    (values (car slot) (cdr slot)))
-						  obj))
-				     (format #f "#<~A: ~{~A~^ ~}>"
-					     ',class-name
-					     (map (lambda (slot)
-						    (list (car slot) (cdr slot)))
-						  obj))))))
-		   (reverse! new-methods)))                      ; the inherited methods, shadowed automatically
+       (set! new-methods
+	     (remove-duplicates
+	      (append (map (lambda (method)
+			     (if (pair? method)
+				 (cons (car method) (cadr method))
+				 (cons method #f)))
+			   ,methods)                     ; the incoming new methods
+		      
+		      ;; add an object->string method for this class (this is already a generic function).
+		      (list (cons 'object->string
+				  (lambda (obj . rest)
+				    (if (and (pair? rest)
+					     (eq? (car rest) :readable))    ; write readably
+					(format #f "(make-~A~{ :~A ~W~^~})"
+						',class-name
+						(map (lambda (slot)
+						       (values (car slot) (cdr slot)))
+						     obj))
+					(format #f "#<~A: ~{~A~^ ~}>"
+						',class-name
+						(map (lambda (slot)
+						       (list (car slot) (cdr slot)))
+						     obj))))))
+		      (reverse! new-methods)))))                  ; the inherited methods, shadowed automatically
 
      (let ((new-class (openlet
                        (apply sublet                             ; the local slots
 			      (sublet                            ; the global slots
 				  (apply inlet                   ; the methods
-					 (reverse new-methods))
+					 new-methods)
 				'class-name ',class-name         ; class-name slot
 				'inherited ,inherited-classes
 				'inheritors ())                  ; classes that inherit from this class
