@@ -209,3 +209,92 @@
 	       (string-append f (substring name i (+ i 2))))))))
 
     (apply combine (map next-file file-names))))
+
+
+#|
+;;; show all timing test overheads
+
+(define (get-overheads file)
+  (with-input-from-file file
+    (lambda ()
+      (let ((overheads ())
+	    (total 0))
+
+	(define (get-overheads-1 file line)
+	  (let ((len (min 20 (length line))))
+	    (do ((i 0 (+ i 1)))
+		((or (= i len)
+		     (not (char-whitespace? (line i))))
+		 (if (and (< i (- len 4))
+			  (char=? (line i) #\.)
+			  (char=? (line (+ i 1)) #\space)
+			  (char=? (line (+ i 2)) #\space)
+			  (char-alphabetic? (line (+ i 3))))
+		     (let ((next-line (read-line)))
+		       (let ((nlen (length next-line)))
+			 (if (char=? (next-line (- nlen 1)) #\{)
+			     (do ((j 0 (+ j 1)))
+				 ((or (= j nlen)
+				      (and (char-numeric? (next-line j))
+					   (let ((cost (string->number-ignoring-commas (substring next-line j (- nlen 3)))))
+					     (set! total (+ total cost))
+					     (set! overheads (cons (list cost (substring line (+ i 3) (min 80 (length line)))) overheads)))))))
+			     (get-overheads-1 file next-line)))))))))
+	
+	(do ((line (read-line) (read-line)))
+	    ((eof-object? line))
+	  (get-overheads-1 file line))
+			     
+	(set! overheads (sort! overheads (lambda (a b) (< (car a) (car b)))))
+	(list-tail overheads (max 10 (- (length overheads) 20)))))))
+
+
+(define file-names '(("make-index.scm" . "/home/bil/motif-snd/v-index85")
+		     ("tmac.scm" . "/home/bil/motif-snd/v-mac85")
+		     ("tpeak.scm" . "/home/bil/motif-snd/v-peak85")
+		     ("tvect.scm" . "/home/bil/motif-snd/v-vect85")
+		     ("teq.scm" . "/home/bil/motif-snd/v-eq85")
+		     ("tfft.scm" . "/home/bil/motif-snd/v-fft85")
+		     ("tref.scm" . "/home/bil/motif-snd/v-ref85")
+		     ("tauto.scm" . "/home/bil/motif-snd/v-auto85")
+		     ("s7test.scm" . "/home/bil/motif-snd/v-test85")
+		     ("tcopy.scm" . "/home/bil/motif-snd/v-cop85")
+		     ("lt.scm" . "/home/bil/motif-snd/v-lt85")
+		     ("tform.scm" . "/home/bil/motif-snd/v-form85")
+		     ("tread.scm" . "/home/bil/motif-snd/v-read85")
+		     ("tmap.scm" . "/home/bil/motif-snd/v-map85")
+		     ("tmat.scm" . "/home/bil/motif-snd/v-mat85")
+		     ("tmisc.scm" . "/home/bil/motif-snd/v-misc85")
+		     ("lg.scm" . "/home/bil/motif-snd/v-lg85")
+		     ("titer.scm" . "/home/bil/motif-snd/v-iter85")
+		     ("tsort.scm" . "/home/bil/motif-snd/v-sort85")
+		     ("tlet.scm" . "/home/bil/motif-snd/v-let85")
+		     ("thash.scm" . "/home/bil/motif-snd/v-hash85")
+		     ("tgen.scm" . "/home/bil/motif-snd/v-gen85")
+		     ("tall.scm" . "/home/bil/motif-snd/v-all85")
+		     ("snd-test.scm" . "/home/bil/motif-snd/v-call85")
+		     ("full-snd-test.scm" . "/home/bil/motif-snd/v-sg85")
+		     ("dup.scm" . "/home/bil/motif-snd/v-dup85")
+		     ("tset.scm" . "/home/bil/motif-snd/v-set85")
+		     ("trec.scm" . "/home/bil/motif-snd/v-rec85")
+		     ("tclo.scm" . "/home/bil/motif-snd/v-clo85")
+		     ("tbig.scm" . "/home/bil/motif-snd/v-big85")
+		     ("tshoot.scm" . "/home/bil/motif-snd/v-shoot85")
+		     ("fbench.scm" . "/home/bil/motif-snd/v-fb85")
+		     ("trclo.scm" . "/home/bil/motif-snd/v-rclo85")
+		     ("tcase.scm" . "/home/bil/motif-snd/v-case85")
+		     ("test-all.scm" . "/home/bil/motif-snd/v-b85")
+		     ("tio.scm" . "/home/bil/motif-snd/v-io85")
+		     ("tgc.scm" . "/home/bil/motif-snd/v-gc85")
+		     ("tnum.scm" . "/home/bil/motif-snd/v-num85")
+		     ("tmock.scm" . "/home/bil/motif-snd/v-mock85")
+		     ("concordance.scm" . "/home/bil/motif-snd/v-str85")
+                     ("tgsl.scm" . "/home/bil/motif-snd/v-gsl85")
+		     ))
+
+(for-each (lambda (file)
+	    (format *stderr* "-------- ~S:\n~{~S~%~^~}~%" (car file) (reverse (get-overheads (cdr file)))))
+	  file-names)
+|#
+
+
