@@ -26,6 +26,7 @@
 (define fft-size (ash 1 17))
 (define little-size 1000000)
 
+
 ;; --------------------------------------------------------------------------------
 (format () "complex fft...~%")
 
@@ -134,12 +135,12 @@
 	(let ((wpr (rationalize (cos theta)))
 	      (wpi (rationalize (sin theta)))
 	      (wr 1)
-	      (wi 0)
-	      (tempr 0)
-	      (tempi 0))
+	      (wi 0))
 	  (do ((ii 0 (+ ii 1)))
 	      ((= ii prev))
-	    (do ((jj 0 (+ jj 1))
+	    (do ((tempr 0)
+		 (tempi 0)
+		 (jj 0 (+ jj 1))
 		 (i ii (+ i mmax))
 		 (j (+ ii prev) (+ j mmax)))
 		((>= jj pow))
@@ -973,9 +974,7 @@
 
 (define (vector-fft rl im n dir)
   (when rl
-    (let ((tempr 0.0)
-	  (tempi 0.0)
-	  (m 0)
+    (let ((m 0)
 	  (j 0)
 	  (ipw (floor (log n 2)))
 	  (prev 1)
@@ -988,7 +987,9 @@
 	  (pw (/ n 2))
 	  (end 0)
 	  (theta (* pi dir)))
-      (do ((i 0 (+ i 1)))
+      (do ((tempr 0.0+i) 
+	   (tempi 0.0+i) 
+	   (i 0 (+ i 1)))
 	  ((= i n))
 	(when (> j i)
 	  (set! tempr (vector-ref rl j))
@@ -1014,7 +1015,9 @@
 	    ((= ii prev))
 	  (set! j (+ ii prev))
 	  (set! end (+ ii (* pw mmax)))
-	  (do ((i ii (+ i mmax)))
+	  (do ((tempr 0.0+i)
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax)))
 	      ((= i end))
 	    (set! tempr (- (* wr (vector-ref rl j)) (* wi (vector-ref im j))))
 	    (set! tempi (+ (* wr (vector-ref im j)) (* wi (vector-ref rl j))))
@@ -1075,11 +1078,10 @@
 (vector-test)
 (clear-and-gc)
 
+
 (define (vector-2d-fft rl n dir)
   (when rl
-    (let ((tempr 0.0)
-	  (tempi 0.0)
-	  (m 0)
+    (let ((m 0)
 	  (j 0)
 	  (ipw (floor (log n 2)))
 	  (prev 1)
@@ -1094,7 +1096,9 @@
 	  (theta (* pi dir))
 	  (rl0 (subvector rl 0 fft-size))
 	  (rl1 (subvector rl fft-size (* 2 fft-size))))
-      (do ((i 0 (+ i 1)))
+      (do ((tempr 0.0+i)
+	   (tempi 0.0+i)
+	   (i 0 (+ i 1)))
 	  ((= i n))
 	(when (> j i)
 	  (set! tempr (vector-ref rl0 j))
@@ -1116,7 +1120,9 @@
 	(set! wpi (sin theta))
 	(set! wr 1.0)
 	(set! wi 0.0)
-	(do ((ii 0 (+ ii 1)))
+	(do ((tempr 0.0+i)
+	     (tempi 0.0+i)
+	     (ii 0 (+ ii 1)))
 	    ((= ii prev))
 	  (set! j (+ ii prev))
 	  (set! end (+ ii (* pw mmax)))
@@ -1214,6 +1220,7 @@
   (clear-and-gc)
   )
 
+
 (define (hash-table-fft rl im n dir)
   (when rl
     (do ((i 0 (+ i 1))
@@ -1237,31 +1244,33 @@
       (do ((mmax 2 (* mmax 2))
 	   (pow (/ n 2) (/ pow 2))
 	   (theta (* pi dir) (* theta 0.5))
-	   (lg 0 (+ lg 1)))
+	   (lg 0 (+ lg 1))
+	   (wpr 0.0)
+	   (wpi 0.0)
+	   (wr 1.0 1.0)
+	   (wi 0.0 0.0)
+	   (wtemp 0.0+i))
 	  ((= lg ipow))
-	(let ((wpr (cos theta))
-	      (wpi (sin theta))
-	      (wr 1.0)
-	      (wi 0.0)
-	      (wtemp 0.0+i)
-	      (tempr 0.0+i)
-	      (tempi 0.0+i))
-	  (do ((ii 0 (+ ii 1)))
-	      ((= ii prev))
-	    (do ((i ii (+ i mmax))
-		 (j (+ ii prev) (+ j mmax))
-		 (jj 0 (+ jj 1)))
-		((>= jj pow))
-	      (set! tempr (- (* wr (hash-table-ref rl j)) (* wi (hash-table-ref im j))))
-	      (set! tempi (+ (* wr (hash-table-ref im j)) (* wi (hash-table-ref rl j))))
-	      (hash-table-set! rl j (- (hash-table-ref rl i) tempr))
-	      (hash-table-set! rl i (+ (hash-table-ref rl i) tempr))
-	      (hash-table-set! im j (- (hash-table-ref im i) tempi))
-	      (hash-table-set! im i (+ (hash-table-ref im i) tempi)))
-	    (set! wtemp wr)
-	    (set! wr (- (* wr wpr) (* wi wpi)))
-	    (set! wi (+ (* wi wpr) (* wtemp wpi))))
-	  (set! prev mmax))))
+	(set! wpr (cos theta))
+	(set! wpi (sin theta))
+	(do ((ii 0 (+ ii 1)))
+	    ((= ii prev))
+	  (do ((tempr 0.0+i)
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax))
+	       (j (+ ii prev) (+ j mmax))
+	       (jj 0 (+ jj 1)))
+	      ((>= jj pow))
+	    (set! tempr (- (* wr (hash-table-ref rl j)) (* wi (hash-table-ref im j))))
+	    (set! tempi (+ (* wr (hash-table-ref im j)) (* wi (hash-table-ref rl j))))
+	    (hash-table-set! rl j (- (hash-table-ref rl i) tempr))
+	    (hash-table-set! rl i (+ (hash-table-ref rl i) tempr))
+	    (hash-table-set! im j (- (hash-table-ref im i) tempi))
+	    (hash-table-set! im i (+ (hash-table-ref im i) tempi)))
+	  (set! wtemp wr)
+	  (set! wr (- (* wr wpr) (* wi wpi)))
+	  (set! wi (+ (* wi wpr) (* wtemp wpi))))
+	(set! prev mmax)))
     rl))
 
 (define (hash-test)
@@ -1301,6 +1310,7 @@
 (hash-test)
 (clear-and-gc)
 
+
 (define (hash-symbol-fft rl im n rl-syms im-syms dir)
   (when rl
     (do ((i 0 (+ i 1))
@@ -1324,31 +1334,33 @@
       (do ((mmax 2 (* mmax 2))
 	   (pow (/ n 2) (/ pow 2))
 	   (theta (* pi dir) (* theta 0.5))
-	   (lg 0 (+ lg 1)))
+	   (lg 0 (+ lg 1))
+	   (wpr 0.0)
+	   (wpi 0.0)
+	   (wr 1.0 1.0)
+	   (wi 0.0 0.0)
+	   (wtemp 0.0+i))
 	  ((= lg ipow))
-	(let ((wpr (cos theta))
-	      (wpi (sin theta))
-	      (wr 1.0)
-	      (wi 0.0)
-	      (tempr 0.0+i)
-	      (tempi 0.0+i)
-	      (wtemp 0.0+i))
-	  (do ((ii 0 (+ ii 1)))
-	      ((= ii prev))
-	    (do ((i ii (+ i mmax))
-		 (j (+ ii prev) (+ j mmax))
-		 (jj 0 (+ jj 1)))
-		((>= jj pow))
-	      (set! tempr (- (* wr (hash-table-ref rl (vector-ref rl-syms j))) (* wi (hash-table-ref im (vector-ref im-syms j)))))
-	      (set! tempi (+ (* wr (hash-table-ref im (vector-ref im-syms j))) (* wi (hash-table-ref rl (vector-ref rl-syms j)))))
-	      (hash-table-set! rl (vector-ref rl-syms j) (- (hash-table-ref rl (vector-ref rl-syms i)) tempr))
-	      (hash-table-set! rl (vector-ref rl-syms i) (+ (hash-table-ref rl (vector-ref rl-syms i)) tempr))
-	      (hash-table-set! im (vector-ref im-syms j) (- (hash-table-ref im (vector-ref im-syms i)) tempi))
-	      (hash-table-set! im (vector-ref im-syms i) (+ (hash-table-ref im (vector-ref im-syms i)) tempi)))
-	    (set! wtemp wr)
-	    (set! wr (- (* wr wpr) (* wi wpi)))
-	    (set! wi (+ (* wi wpr) (* wtemp wpi))))
-	  (set! prev mmax))))
+	(set! wpr (cos theta))
+	(set! wpi (sin theta))
+	(do ((ii 0 (+ ii 1)))
+	    ((= ii prev))
+	  (do ((tempr 0.0+i)
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax))
+	       (j (+ ii prev) (+ j mmax))
+	       (jj 0 (+ jj 1)))
+	      ((>= jj pow))
+	    (set! tempr (- (* wr (hash-table-ref rl (vector-ref rl-syms j))) (* wi (hash-table-ref im (vector-ref im-syms j)))))
+	    (set! tempi (+ (* wr (hash-table-ref im (vector-ref im-syms j))) (* wi (hash-table-ref rl (vector-ref rl-syms j)))))
+	    (hash-table-set! rl (vector-ref rl-syms j) (- (hash-table-ref rl (vector-ref rl-syms i)) tempr))
+	    (hash-table-set! rl (vector-ref rl-syms i) (+ (hash-table-ref rl (vector-ref rl-syms i)) tempr))
+	    (hash-table-set! im (vector-ref im-syms j) (- (hash-table-ref im (vector-ref im-syms i)) tempi))
+	    (hash-table-set! im (vector-ref im-syms i) (+ (hash-table-ref im (vector-ref im-syms i)) tempi)))
+	  (set! wtemp wr)
+	  (set! wr (- (* wr wpr) (* wi wpi)))
+	  (set! wi (+ (* wi wpr) (* wtemp wpi))))
+	(set! prev mmax)))
     rl))
 
 (define (hash-symbol-test)
@@ -1423,31 +1435,33 @@
       (do ((mmax 2 (* mmax 2))
 	   (pow (/ n 2) (/ pow 2))
 	   (theta (* pi dir) (* theta 0.5))
-	   (lg 0 (+ lg 1)))
+	   (lg 0 (+ lg 1))
+	   (wpr 0.0)
+	   (wpi 0.0)
+	   (wr 1.0 1.0)
+	   (wi 0.0 0.0)
+	   (wtemp 0.0+i))
 	  ((= lg ipow))
-	(let ((wpr (cos theta))
-	      (wpi (sin theta))
-	      (wr 1.0)
-	      (wi 0.0)
-	      (tempr 0.0+i)
-	      (tempi 0.0+i)
-	      (wtemp 0.0+i))
-	  (do ((ii 0 (+ ii 1)))
-	      ((= ii prev))
-	    (do ((i ii (+ i mmax))
-		 (j (+ ii prev) (+ j mmax))
-		 (jj 0 (+ jj 1)))
-		((>= jj pow))
-	      (set! tempr (- (* wr (let-ref rl (vector-ref rl-syms j))) (* wi (let-ref im (vector-ref im-syms j)))))
-	      (set! tempi (+ (* wr (let-ref im (vector-ref im-syms j))) (* wi (let-ref rl (vector-ref rl-syms j)))))
-	      (let-set! rl (vector-ref rl-syms j) (- (let-ref rl (vector-ref rl-syms i)) tempr))
-	      (let-set! rl (vector-ref rl-syms i) (+ (let-ref rl (vector-ref rl-syms i)) tempr))
-	      (let-set! im (vector-ref im-syms j) (- (let-ref im (vector-ref im-syms i)) tempi))
-	      (let-set! im (vector-ref im-syms i) (+ (let-ref im (vector-ref im-syms i)) tempi)))
-	    (set! wtemp wr)
-	    (set! wr (- (* wr wpr) (* wi wpi)))
-	    (set! wi (+ (* wi wpr) (* wtemp wpi))))
-	  (set! prev mmax))))
+	(set! wpr (cos theta))
+	(set! wpi (sin theta))
+	(do ((ii 0 (+ ii 1)))
+	    ((= ii prev))
+	  (do ((tempr 0.0+i)
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax))
+	       (j (+ ii prev) (+ j mmax))
+	       (jj 0 (+ jj 1)))
+	      ((>= jj pow))
+	    (set! tempr (- (* wr (let-ref rl (vector-ref rl-syms j))) (* wi (let-ref im (vector-ref im-syms j)))))
+	    (set! tempi (+ (* wr (let-ref im (vector-ref im-syms j))) (* wi (let-ref rl (vector-ref rl-syms j)))))
+	    (let-set! rl (vector-ref rl-syms j) (- (let-ref rl (vector-ref rl-syms i)) tempr))
+	    (let-set! rl (vector-ref rl-syms i) (+ (let-ref rl (vector-ref rl-syms i)) tempr))
+	    (let-set! im (vector-ref im-syms j) (- (let-ref im (vector-ref im-syms i)) tempi))
+	    (let-set! im (vector-ref im-syms i) (+ (let-ref im (vector-ref im-syms i)) tempi)))
+	  (set! wtemp wr)
+	  (set! wr (- (* wr wpr) (* wi wpi)))
+	  (set! wi (+ (* wi wpr) (* wtemp wpi))))
+	(set! prev mmax)))
     rl))
 
 (define (let-test)
@@ -1493,14 +1507,13 @@
 (let-test)
 (clear-and-gc)
 
+
 (format () "~%sundries...~%")
 ;(set! fft-size (ash 1 15))
 
 (define (string-fft rl im n dir)
   (when rl
-    (let ((tempr 0.0+i)
-	  (tempi 0.0+i)
-	  (m 0)
+    (let ((m 0)
 	  (j 0)
 	  (ipw (floor (log n 2)))
 	  (prev 1)
@@ -1513,7 +1526,9 @@
 	  (pw (/ n 2))
 	  (end 0)
 	  (theta (* pi dir)))
-      (do ((i 0 (+ i 1)))
+      (do ((tempr 0.0+i)
+	   (tempi 0.0+i)
+	   (i 0 (+ i 1)))
 	  ((= i n))
 	(when (> j i)
 	  (set! tempr (string->number (vector-ref rl j)))
@@ -1539,7 +1554,9 @@
 	    ((= ii prev))
 	  (set! j (+ ii prev))
 	  (set! end (+ ii (* pw mmax)))
-	  (do ((i ii (+ i mmax)))
+	  (do ((tempr 0.0+i) 
+	       (tempi 0.0+i) 
+	       (i ii (+ i mmax)))
 	      ((= i end))
 	    (set! tempr (- (* wr (string->number (vector-ref rl j))) (* wi (string->number (vector-ref im j)))))
 	    (set! tempi (+ (* wr (string->number (vector-ref im j))) (* wi (string->number (vector-ref rl j)))))
@@ -1600,14 +1617,13 @@
     (fill! fvi #f)))
 (string-test)
 
+
 (let-temporarily (((*s7* 'float-format-precision) 48))
 (format *stderr* "~%radix...~%")
 (define radix 2)
 (define (string-radix-fft rl im n dir)
   (when rl
-    (let ((tempr 0.0+i)
-	  (tempi 0.0+i)
-	  (m 0)
+    (let ((m 0)
 	  (j 0)
 	  (ipw (floor (log n 2)))
 	  (prev 1)
@@ -1620,7 +1636,9 @@
 	  (pw (/ n 2))
 	  (end 0)
 	  (theta (* pi dir)))
-      (do ((i 0 (+ i 1)))
+      (do ((tempr 0.0+i)
+	   (tempi 0.0+i)
+	   (i 0 (+ i 1)))
 	  ((= i n))
 	(when (> j i)
 	  (set! tempr (string->number (vector-ref rl j) radix))
@@ -1646,7 +1664,9 @@
 	    ((= ii prev))
 	  (set! j (+ ii prev))
 	  (set! end (+ ii (* pw mmax)))
-	  (do ((i ii (+ i mmax)))
+	  (do ((tempr 0.0+i) 
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax)))
 	      ((= i end))
 	    (set! tempr (- (* wr (string->number (vector-ref rl j) radix)) (* wi (string->number (vector-ref im j) radix))))
 	    (set! tempi (+ (* wr (string->number (vector-ref im j) radix)) (* wi (string->number (vector-ref rl j) radix))))
@@ -1709,11 +1729,10 @@
 
 (clear-and-gc)
 
+
 (define (port-fft rl im n dir)
   (when rl
-    (let ((tempr 0.0+i)
-	  (tempi 0.0+i)
-	  (m 0)
+    (let ((m 0)
 	  (j 0)
 	  (ipw (floor (log n 2)))
 	  (prev 1)
@@ -1726,7 +1745,9 @@
 	  (pw (/ n 2))
 	  (end 0)
 	  (theta (* pi dir)))
-      (do ((i 0 (+ i 1)))
+      (do ((tempr 0.0+i) 
+	   (tempi 0.0+i) 
+	   (i 0 (+ i 1)))
 	  ((= i n))
 	(when (> j i)
 	  (set! tempr (with-input-from-string (vector-ref rl j) read))
@@ -1752,7 +1773,9 @@
 	    ((= ii prev))
 	  (set! j (+ ii prev))
 	  (set! end (+ ii (* pw mmax)))
-	  (do ((i ii (+ i mmax)))
+	  (do ((tempr 0.0+i) 
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax)))
 	      ((= i end))
 	    (set! tempr (- (* wr (with-input-from-string (vector-ref rl j) read)) (* wi (with-input-from-string (vector-ref im j) read))))
 	    (set! tempi (+ (* wr (with-input-from-string (vector-ref im j) read)) (* wi (with-input-from-string (vector-ref rl j) read))))
@@ -1814,8 +1837,20 @@
 (port-test)
 
 
+;(define-expansion (new-value val)
+;  `(make-vector (make-list (+ 1 (random 4)) (+ 1 (random 4))) ,val))
+
+(define lists (make-vector 16))
+(do ((i 0 (+ i 1))
+     (j 1)
+     (k 1))
+    ((= i 16))
+  (set! (lists i) (make-list k j))
+  (set! j (+ j 1))
+  (when (= j 5) (set! j 1) (set! k (+ k 1))))
+
 (define-expansion (new-value val)
-  `(make-vector (make-list (+ 1 (random 4)) (+ 1 (random 4))) ,val))
+  `(make-vector (vector-ref lists (random 16)) ,val))
 
 (define-expansion (old-value ref)
   `((subvector ,ref 0 1) 0))
@@ -1840,22 +1875,23 @@
       (do ((lg 0 (+ lg 1))
 	   (mmax 2 (* mmax 2))
 	   (pow (/ n 2) (/ pow 2))
-	   (theta (complex 0.0 (* pi dir)) (* theta 0.5)))
+	   (theta (complex 0.0 (* pi dir)) (* theta 0.5))
+	   (wpc 0.0)
+	   (wc 1.0 1.0))
 	  ((= lg ipow))
-	(let ((wpc (exp theta))
-	      (wc 1.0)
-	      (tc 0.0))
-	  (do ((ii 0 (+ ii 1)))
-	      ((= ii prev))
-	    (do ((jj 0 (+ jj 1))
-		 (i ii (+ i mmax))
-		 (j (+ ii prev) (+ j mmax)))
-		((>= jj pow))
-	      (set! tc (* wc (old-value (vector-ref data j))))
-	      (vector-set! data j (new-value (- (old-value (vector-ref data i)) tc)))
-	      (vector-set! data i (new-value (+ (old-value (vector-ref data i)) tc))))
-	    (set! wc (* wc wpc)))
-	  (set! prev mmax))))
+	(set! wpc (exp theta))
+	(do ((ii 0 (+ ii 1)))
+	    ((= ii prev))
+	  (do ((tc 0.0+i)
+	       (jj 0 (+ jj 1))
+	       (i ii (+ i mmax))
+	       (j (+ ii prev) (+ j mmax)))
+	      ((>= jj pow))
+	    (set! tc (* wc (old-value (vector-ref data j))))
+	    (vector-set! data j (new-value (- (old-value (vector-ref data i)) tc)))
+	    (vector-set! data i (new-value (+ (old-value (vector-ref data i)) tc))))
+	  (set! wc (* wc wpc)))
+	(set! prev mmax)))
     data))
 
 (define (complex-2d-checker fvr)
@@ -1893,8 +1929,9 @@
     (fill! fvr #f)))
 (complex-2d-test)
 
+
 (clear-and-gc)
-(set! fft-size (ash 1 17))
+;(set! fft-size (ash 1 17))
 (define (complex-hash-fft data n dir)
   (when data
     (do ((i 0 (+ i 1))
@@ -1974,9 +2011,7 @@
 
 (define (iterator-vector-fft rl im n dir)
   (when rl
-    (let ((tempr 0.0+i)
-	  (tempi 0.0+i)
-	  (m 0)
+    (let ((m 0)
 	  (j 0)
 	  (ipw (floor (log n 2)))
 	  (prev 1)
@@ -1991,7 +2026,9 @@
 	  (theta (* pi dir))
 	  (tempbr #f)
 	  (tempbi #f))
-      (do ((i 0 (+ i 1)))
+      (do ((tempr 0.0+i) 
+	   (tempi 0.0+i)
+	   (i 0 (+ i 1)))
 	  ((= i n))
 	(when (> j i)
 	  (set! tempbr (vector-ref rl j))
@@ -2017,7 +2054,9 @@
 	    ((= ii prev))
 	  (set! j (+ ii prev))
 	  (set! end (+ ii (* pw mmax)))
-	  (do ((i ii (+ i mmax)))
+	  (do ((tempr 0.0+i)
+	       (tempi 0.0+i)
+	       (i ii (+ i mmax)))
 	      ((= i end))
 	    (set! tempr (- (* wr (iterate (vector-ref rl j))) (* wi (iterate (vector-ref im j)))))
 	    (set! tempi (+ (* wr (iterate (vector-ref im j))) (* wi (iterate (vector-ref rl j)))))
