@@ -4,12 +4,6 @@
 (define g-size 1000000)
 (define kf-size 30)
 (define k100-size 10000)
-#|
-(define d1-size 0)
-(define g-size 0)
-(define kf-size 30)
-(define k100-size 0)
-|#
 
 (define* (f0 a b)
   (display b #f))
@@ -24,7 +18,7 @@
   (display 3 #f))
 
 (define* (f4)
-  (apply procedure? (list +)))
+  (apply + (list 1 2)))
 
 (define* (f5 (a 1))
   (apply + (list a 2)))
