@@ -2,15 +2,16 @@
 (define size 1024)
 
 (define (b_fft areal aimag)
-  (let ((ar 0)
-        (ai 0)
-        (i 1)
+  (let ((i 1)
+	(n (- (length areal) 1))
         (j 1)
         (k 0)
         (m 0)
-        (n 0)
+	(ar areal)
+        (ai aimag)
         (le 1)
-        (le1 0) (le2 0)
+        (le1 0) 
+	(le2 0)
         (ip 0)
         (nv2 0)
         (nm1 0)
@@ -21,10 +22,6 @@
         (tr 0.0)
         (ti 0.0))
     ;; initialize
-    (set! ar areal)
-    (set! ai aimag)
-    (set! n (length ar))
-    (set! n (- n 1))
     (set! nv2 (quotient n 2))
     (set! nm1 (- n 1))
     
