@@ -313,26 +313,14 @@
   (fe13 1))
 
 
+;;; --------------------------------------------------------------------------------
 ;;; this is a revision of some code posted in comp.lang.lisp by melzzzzz for euler project 512
 
-#|
-(define (make-boolean-vector n)
-  (make-int-vector (ceiling (/ n 63))))
-
-(define-expansion (boolean-vector-ref v n)
-  `(logbit? (int-vector-ref ,v (quotient ,n 63)) (remainder ,n 63)))
-
-(define-expansion (boolean-vector-set! v n)
-  `(int-vector-set! ,v (quotient ,n 63)
-		    (logior (int-vector-ref ,v (quotient ,n 63))
-			    (ash 1 (remainder ,n 63)))))
-|#
-;;; this is slightly faster (using int-vector is better for the largest cases)
+;;; using int-vector is better for the largest cases
 (define (make-boolean-vector n) (make-vector n #f))
 (define boolean-vector-ref vector-ref)
 (define-expansion (boolean-vector-set! v j) `(vector-set! ,v ,j #t))
 
-
 (define (odd-get n)
   (let* ((visited-range (+ (ash n -1) 1))
 	 (visited (make-boolean-vector visited-range))
@@ -386,7 +374,307 @@
 ;;; (getr 50000000)   506605921933035    6 
 ;;; (getr 500000000)  50660591862310323  67 (32M, mutable_do)
 
+;;; --------------------------------------------------------------------------------
+;;; some coverage cases
+
+(define fsize 200000)
+(define (f1 lst)
+  (for-each (lambda (p)
+	      (if (integer? p)
+		  (display 'oops)))
+	    lst))
+
+(define lst (make-list fsize ()))
+(f1 lst)
+
+(define (f2 v)
+  (for-each (lambda (p)
+	      (if (integer? p)
+		  (display 'oops)))
+	    v))
+
+(define fv (make-float-vector fsize 1.0))
+(f2 fv)
+
+(define (f3 v)
+  (for-each (lambda (p)
+	      (if (pair? p)
+		  (display 'oops)))
+	    v))
+
+(define iv (make-int-vector fsize 1))
+(f3 iv)
+
+(define (f4 v)
+  (for-each (lambda (p)
+	      (if (integer? p)
+		  (display 'oops)))
+	    v))
+
+(define nv (make-vector fsize "abc"))
+(f4 nv)
+
+(define (f5 lst)
+  (for-each (lambda (p)
+	      (if (integer? p)
+		  (throw 'oops p)))
+	    lst))
+(f5 lst)
+
+(define (f11 lst)
+  (map (lambda (p)
+	 (if (integer? p)
+	     (display 'oops)))
+       lst))
+(f11 lst)
+
+(define (f12 v)
+  (map (lambda (p)
+	 (if (integer? p)
+	     (display 'oops)))
+       v))
+(f12 fv)
+
+(define (f13 v)
+  (map (lambda (p)
+	 (if (pair? p)
+	     (display 'oops)))
+       v))
+(f13 iv)
+
+(define (f14 v)
+  (map (lambda (p)
+	 (if (integer? p)
+	     (display 'oops)))
+       v))
+(f14 nv)
+
+(define (f15 lst)
+  (map (lambda (p)
+	 (if (integer? p)
+	     (throw 'oops p)))
+       lst))
+(f15 lst)
+
+
+
+(define gsize 1000000)
+
+(define (g1 lst)
+  (for-each (lambda* (p)
+	      (if (integer? p)
+		  (display 'oops)))
+	    lst))
+
+(define glst (make-list gsize ()))
+(g1 glst)
+
+(define (g2 v)
+  (for-each (lambda* (p)
+	      (if (integer? p)
+		  (display 'oops)))
+	    v))
+
+(define gfv (make-float-vector gsize 1.0))
+(g2 gfv)
+
+(define (g3 v)
+  (for-each (lambda* (p)
+	      (if (pair? p)
+		  (display 'oops)))
+	    v))
+
+(define giv (make-int-vector gsize 1))
+(g3 giv)
+
+(define (g4 v)
+  (for-each (lambda* (p)
+	      (if (integer? p)
+		  (display 'oops)))
+	    v))
+
+(define gv (make-vector gsize ()))
+(g4 gv)
+
+(define (g5 lst)
+  (for-each (lambda* (p)
+	      (if (integer? p)
+		  (throw 'oops p)))
+	    lst))
+(g5 glst)
+
+(define (g11 lst)
+  (map (lambda* (p)
+	 (if (integer? p)
+	     (display 'oops)
+             0))
+       lst))
+(g11 glst)
+
+(define (g12 v)
+  (map (lambda* (p)
+	 (if (integer? p)
+	     (display 'oops)
+             0))
+       v))
+(g12 gfv)
+
+(define (g13 v)
+  (map (lambda* (p)
+	 (if (pair? p)
+	     (display 'oops)
+             0))
+       v))
+(g13 giv)
+
+(define (g14 v)
+  (map (lambda* (p)
+	 (if (integer? p)
+	     (display 'oops)
+             0))
+       v))
+(g14 gv)
+
+(define (g15 lst)
+  (map (lambda* (p)
+	 (if (integer? p)
+	     (throw 'oops p)
+             0))
+       lst))
+(g15 glst)
+
+(define (g6 lst)
+  (for-each (lambda (p)
+	      (if (integer? p)
+		  (display 'oops))
+	      (if (pair? p)
+		  (display 'oops)))
+	    lst))
+(g6 gv)
+
+(define (g6 lst)
+  (for-each (lambda (p)
+	      (for-each (lambda (q)
+			  (if (integer? q)
+			      (display 'oops)))
+			p))
+	    lst))
+
+(define glst1 (make-list 100 (make-list 100 #\a)))
+(g6 glst1)
+
+(define (g16 lst)
+  (map (lambda (p)
+	 (map (lambda (q)
+		(if (integer? q)
+		    (display 'oops)
+		    0))
+	      p))
+       lst))
+(g16 glst1)
+
+(define fstr (make-string gsize #\a))
+(define (f7 str)
+  (for-each (lambda (p)
+	      (if (not (char? p))
+		  (display 'oops)))
+	    str))
+(f7 fstr)
+
+(define (f17 str)
+  (map (lambda (p)
+	 (if (not (char? p))
+	     (display 'oops)
+	     p))
+       str))
+(f17 fstr)
+
+
+(define (f21 lst)
+  (for-each (lambda (p q)
+	      (unless (eq? p q)
+		(display 'oops)))
+	    lst lst))
+(f21 lst)
+
+
+(define (f22 v)
+  (for-each (lambda (p q)
+	      (if (not (= p q))
+		  (display 'oops)))
+	    v v))
+(f22 fv)
+
+
+(define (f23 str)
+  (for-each (lambda (p q)
+	      (if (not (char=? p q))
+		  (display 'oops)))
+	    str str))
+(f23 fstr)
+
+
+(define (f24 v)
+  (for-each (lambda (p q)
+	      (unless (string=? p q)
+		(display 'oops)))
+	    v v))
+(f24 nv)
+
+;; if float-vector arg and only ref is in opt_d context, no need for make_real
+;;   i.e. 2 args even cell_opt outer, is ref b_dd or equivalent?
+
+
+(define hsize 75000)
+(define (f25 v)
+  (do ((i 0 (+ i 1)))
+      ((= i hsize))
+    (for-each (lambda (p)
+		(unless (positive? p)
+		  (display 'oops)))
+	      v)))
+
+(define fgv (make-float-vector 100 1.0))
+(f25 fgv)
+
+
+(define (f26 v1 v2)
+  (for-each (lambda (p q)
+	      (if (not (= p q))
+		  (display 'oops))) ;(format *stderr* "f2: ~S ~S~%" p q)))
+	    v1 v2))
+(f26 fv iv)
+
+(define (f27 lst v)
+  (for-each (lambda (p q)
+	      (if (equal? p q)
+		  (display 'oops))) ;(format *stderr* "f1: ~S ~S~%" p q)))
+	    lst v))
+(f27 lst nv)
+
+(define (f28)
+  (do ((p lst (cdr p))
+       (i 0 (+ i 1)))
+      ((null? p))
+    (set-car! p i))
+  (map + lst lst))
+(f28)
+
+(define (f29)
+  (let ((str (make-string 100 #\a)))
+    (set! (str 50) #\b)
+    (do ((i 0 (+ i 1)))
+	((= i 30000))
+      (map char->integer str))))
+(f29)
+
+(define (f30)
+  (do ((i 0 (+ i 1)))
+      ((= i 30))
+    (map pair? lst)))
+(f30)
+
+(newline)
 (when (> (*s7* 'profile) 0)
   (show-profile 200))
 (exit)
-
