@@ -45,6 +45,9 @@
 		     ("tari.scm" . "v-ari")
 		     ("texit.scm" . "v-exit")
 		     ("tleft.scm" . "v-left")
+		     ("tobj.scm" . "v-obj")
+		     ("timp.scm" . "v-imp")
+		     ("tlamb.scm" . "v-lamb")
 		     ))
 
 (define (last-callg)
@@ -79,44 +82,47 @@
 
    (list (list "repl" "tpeak.scm")
 	 (list "repl" "tref.scm")
-	 (list "repl" "tauto.scm")
 	 (list "snd -noinit" "make-index.scm")
 	 (list "repl" "tmock.scm")
+	 (list "repl" "texit.scm")
 	 (list "repl" "tvect.scm")
 	 (list "repl" "s7test.scm")
-	 (list "repl" "texit.scm")
 	 (list "repl" "lt.scm")
-	 (list "repl" "tform.scm")
+	 (list "repl" "timp.scm")
 	 (list "repl" "tread.scm")
-	 (list "repl" "tmac.scm")
-	 (list "repl" "fbench.scm")
 	 (list "repl" "trclo.scm")
+	 (list "repl" "fbench.scm")
+	 (list "repl" "dup.scm")
 	 (list "repl" "tmat.scm")
 	 (list "repl" "tcopy.scm")
-	 (list "repl" "dup.scm")
+	 (list "repl" "tauto.scm")
 	 (list "repl" "titer.scm")
 	 (list "repl" "tsort.scm")
-	 (list "repl" "tset.scm")
+	 (list "repl" "tmac.scm")
 	 (list "repl" "tload.scm")
+	 (list "repl" "tset.scm")
 	 (list "repl" "teq.scm")
 	 (list "repl" "tio.scm")
+	 (list "repl" "tobj.scm")
+	 (list "repl" "tlamb.scm")
 	 (list "repl" "tclo.scm")
-	 (list "repl" "tlet.scm")
 	 (list "repl" "tcase.scm")
+	 (list "repl" "tlet.scm")
 	 (list "repl" "tmap.scm")
 	 (list "repl" "tfft.scm")
 	 (list "repl" "tshoot.scm")
+	 (list "repl" "tform.scm")
 	 (list "repl" "tnum.scm")
 	 (list "repl" "concordance.scm")
+	 (list "repl" "tmisc.scm")
 	 (list "repl" "tgsl.scm")
 	 (list "repl" "trec.scm")
-	 (list "repl" "tmisc.scm")
 	 (list "repl" "tlist.scm")
 	 (reader-cond ((not (provided? 'gmp)) (list "repl" "tari.scm")))
 	 (list "repl" "tleft.scm")
 	 (list "repl" "tgc.scm")
-	 (list "repl" "thash.scm")
 	 (list "repl" "cb.scm")
+	 (list "repl" "thash.scm")
 	 (list "snd -noinit" "tgen.scm")    ; repl here + cload sndlib was slower
 	 (list "snd -noinit" "tall.scm")
 	 (list "snd -l" "snd-test.scm")
